-----------------------------------------------------------------------------------------------
-- Client Lua Script for StatMeter
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- StatMeter Module Definition
-----------------------------------------------------------------------------------------------
local StatMeter = {} 
local version = 1.1
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function StatMeter:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
	
    -- initialize variables here
	o.tItems = {}
	o.tToggles = {}
	-- 
	o.tConfig = {}
	o.tConfig.cToggle = 0
	o.tConfig.IsVisible = true
	o.tConfig.Position = -- Default position
 	{
  		Bottom = 242,
  		Left = 8,
  		Right = 200,
  		Top = 8,
 	}
	o.tConfig.Version = 1
	
	
    return o
end

function StatMeter:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- StatMeter OnLoad
-----------------------------------------------------------------------------------------------
function StatMeter:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("StatMeter.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- StatMeter OnDocLoaded
-----------------------------------------------------------------------------------------------
function StatMeter:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "Background", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
		else
			--closed by default
			self.wndMain:Show(false, true)
			
			--Print("Setting Position")
			--Print(self.tConfig.Position.Left)
			if self.tConfig ~= nil then
				if self.tConfig.Position ~= nil then
					self.wndMain:SetAnchorOffsets(
					self.tConfig.Position.Left,
					self.tConfig.Position.Top,
		      		self.tConfig.Position.Right,
		      		self.tConfig.Position.Bottom)
				end
			end
		end
		StatMeter.wndMain = self.wndMain
	    --self.wndMain:Show(true, true)

		
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("statmeter", "OnStatMeterOn", self)
		Apollo.RegisterSlashCommand("stathud", "OnStatMeterOn", self)
		Apollo.RegisterSlashCommand("Stathud", "OnStatMeterOn", self)
		Apollo.RegisterSlashCommand("StatHUD", "OnStatMeterOn", self)
		
		-- Do additional Addon initialization here
		--Timers
		Apollo.CreateTimer("UpdateTimer", 0.90, true, self)
		Apollo.RegisterTimerHandler("UpdateTimer", "OnUpdateTimer", self)
		
		
		if self.tConfig ~= nil and self.tConfig.IsVisible == true then
    		self:OnShow()
   		end
		
		
		--item list
		self.wndItemList = self.wndMain:FindChild("StatContainer")
		self:PopulateSpecificStatList(self.tConfig.cToggle)
	end
end

-----------------------------------------------------------------------------------------------
-- StatMeter Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/stathud"
function StatMeter:OnStatMeterOn()
	if self.wndMain:IsShown() == true then
		Print("StatHUD Hidden")
		self:OnHide()
	else
		Print("StatHUD Shown")
		self:OnShow()
	end
end

function StatMeter:OnShow()
	self.tConfig.IsVisible = true
	self.wndMain:Show(true, false)
	
end

function StatMeter:OnHide()
	self.tConfig.IsVisible = false
	self.wndMain:Show(false, false)
end

function StatMeter:OnToggle()
	local rollToggleWnd = self.wndMain:FindChild("RollToggle")
	if self.tConfig.cToggle == 5 then 
		self.tConfig.cToggle = 0
	else
		self.tConfig.cToggle = self.tConfig.cToggle + 1
	end
	if self.tConfig.cToggle == 0 then rollToggleWnd:SetText("DPS") end
	if self.tConfig.cToggle == 1 then rollToggleWnd:SetText("Healer") end
	if self.tConfig.cToggle == 2 then rollToggleWnd:SetText("Tank") end
	if self.tConfig.cToggle == 3 then rollToggleWnd:SetText("PVP DPS") end
	if self.tConfig.cToggle == 4 then rollToggleWnd:SetText("PVP Healer") end
	if self.tConfig.cToggle == 5 then rollToggleWnd:SetText("PVP Tank") end
	--Print(self.tConfig.cToggle)
	self:DestroyItemList()
	self:PopulateSpecificStatList(self.tConfig.cToggle)
end

function StatMeter:OnToggleBack()
	local rollToggleWnd = self.wndMain:FindChild("RollToggle")
	if self.tConfig.cToggle == 0 then 
		self.tConfig.cToggle = 5
	else
		self.tConfig.cToggle = self.tConfig.cToggle - 1
	end
	if self.tConfig.cToggle == 0 then rollToggleWnd:SetText("DPS") end
	if self.tConfig.cToggle == 1 then rollToggleWnd:SetText("Healer") end
	if self.tConfig.cToggle == 2 then rollToggleWnd:SetText("Tank") end
	if self.tConfig.cToggle == 3 then rollToggleWnd:SetText("PVP DPS") end
	if self.tConfig.cToggle == 4 then rollToggleWnd:SetText("PVP Healer") end
	if self.tConfig.cToggle == 5 then rollToggleWnd:SetText("PVP Tank") end
	--Print(self.tConfig.cToggle)
	self:DestroyItemList()
	self:PopulateSpecificStatList(self.tConfig.cToggle)
end

function StatMeter:GetAllStats()
	local player = GameLib:GetPlayerUnit()
	if player == nil then return
	else
		local playerstats = {		
			{id = 1, name = "AP", value = self:round(player:GetAssaultPower(), 0)},
			{id = 2, name = "SP", value = self:round(player:GetSupportPower(), 0)},
			{id = 3, name = "Critical", value = (self:round((player:GetCritChance().nAmount), 2).."%")},
			{id = 4, name = "StrkThru", value = (self:round((player:GetStrikethroughChance().nAmount), 2).."%")},
			{id = 5, name = "PVPOff", value = (self:round((player:GetPvPDamageI()), 2).."%")},
			{id = 6, name = "PVPDef", value = (self:round((player:GetPvPDefenseI()), 2).."%")},
			{id = 7, name = "Deflect", value = (self:round((player:GetDeflectChance().nAmount), 2).."%")},
			{id = 8, name = "DeflectC", value = (self:round((player:GetDeflectCritChance().nAmount), 2).."%")},
			{id = 9, name = "Armor", value = player:GetUnitProperties().Armor.fValue},
			{id = 10, name = "PVPHeal", value = (self:round((player:GetPvPHealingI()), 2).."%")},
			{id = 11, name = "HealOut", value = (self:round((player:GetUnitProperties().HealingMultiplierOutgoing.fValue * 100), 2).."%")},
			{id = 12, name = "Speed", value = self:round(player:GetUnitProperties().MoveSpeedMultiplier.fValue, 3)},
			{id = 13, name = "CritSev", value = (self:round((player:GetCritSeverity().nAmount), 2).."%")},
			{id = 14, name = "MultiHit", value = (self:round((player:GetMultiHitChance().nAmount), 2).."%")},
			{id = 15, name = "Intensity", value = (self:round((player:GetIntensity().nAmount), 2).."%")},
			{id = 15, name = "FocusReg", value = (self:round((player:GetFocusRegenInCombat().nAmount), 2).."%")},
			{id = 15, name = "CritMit", value = (self:round((player:GetCriticalMitigation().nAmount), 2).."%")}
			}
		return playerstats
	end
end

function StatMeter:GetSupportPower()
	local player = GameLib:GetPlayerUnit()
	local playersp = StatMeter:round(player:GetSupportPower(), 0)
	return playersp
end

function StatMeter:UpdateSupportPower(n)
	local wnd = StatMeter.wndMain
	local wndsp = wnd:FindChild("SupportPower")
	wndsp:SetText(n)
end

function StatMeter:GetAssaultPower()
	local player = GameLib:GetPlayerUnit()
	local playersp = StatMeter:round(player:GetAssaultPower(), 0)
	return playersp
end

function StatMeter:UpdateAssaultPower(n)
	local wnd = StatMeter.wndMain
	local wndap = wnd:FindChild("AssaultPower")
	wndap:SetText(n)
end

function StatMeter:Update()
	StatMeter:UpdateSupportPower(StatMeter:GetSupportPower())
	StatMeter:UpdateAssaultPower(StatMeter:GetAssaultPower())
end

function StatMeter:OnUpdateTimer()
	local stats = self:GetAllStats()
	local wndRunSpeed = self.wndMain:FindChild("RunSpeed")
	--local wndRunSpeedSprite = wndRunSpeed:GetSprite()
	--Print("update")
	if stats ~= nil then 
		--Print("IF")		
		--if wndRunSpeed ~= nil then
			--for key,stat in pairs(stats) do
			--	if stat.name == "Speed" then 
				--	wndRunSpeed:SetText("Speed: ".. stat.value) 
					--if stat.value <= 1 and wndRunSpeedSprite ~= "CRB_Tooltips:sprTooltip_SquareFrame_White" then
					--	wndRunSpeed:SetSprite("CRB_Tooltips:sprTooltip_SquareFrame_UnitRed")
					--elseif stat.value >=1 then 
					--	wndRunSpeed:SetSprite("CRB_Tooltips:sprTooltip_SquareFrame_White")
					--end
				--end
			--end
		--end
		for idx,wnd in pairs(self.tItems) do
			local wndItemText = wnd:GetText()
			for idx,stat in pairs(stats) do
				--Print(wndItemText)
				if string.match(wndRunSpeed:GetText(), stat.name) == "Speed" then 
					wndRunSpeed:SetText("Speed: ".. stat.value) 
					if stat.value < 1 and wndRunSpeedSprite ~= "CRB_Tooltips:sprTooltip_SquareFrame_White" then
						wndRunSpeed:SetSprite("CRB_Tooltips:sprTooltip_SquareFrame_UnitRed")
					elseif stat.value >=1 then 
						wndRunSpeed:SetSprite("CRB_Tooltips:sprTooltip_SquareFrame_White")
					end
				end
				if string.match(wndItemText, stat.name) == "SP" then wnd:SetText(stat.name .. ": " .. stat.value) end
				if string.match(wndItemText, stat.name) == "AP" then wnd:SetText(stat.name .. ": " .. stat.value) end
				if string.match(wndItemText, stat.name) == "Critical" then wnd:SetText(stat.name .. ": " .. stat.value) end
				if string.match(wndItemText, stat.name) == "StrkThru" then wnd:SetText(stat.name .. ": " .. stat.value) end
				if string.match(wndItemText, stat.name) == "PVPOff" then wnd:SetText(stat.name .. ": " .. stat.value) end
				if string.match(wndItemText, stat.name) == "PVPDef" then wnd:SetText(stat.name .. ": " .. stat.value) end
				if string.match(wndItemText, stat.name) == "Deflect" then wnd:SetText(stat.name .. ": " .. stat.value) end
				if string.match(wndItemText, stat.name) == "DeflectC" then wnd:SetText(stat.name .. ": " .. stat.value) end
				if string.match(wndItemText, stat.name) == "Armor" then wnd:SetText(stat.name .. ": " .. stat.value) end
				if string.match(wndItemText, stat.name) == "PVPHeal" then wnd:SetText(stat.name .. ": " .. stat.value) end
				if string.match(wndItemText, stat.name) == "HealOut" then wnd:SetText(stat.name .. ": " .. stat.value) end
				if string.match(wndItemText, stat.name) == "CritSev" then wnd:SetText(stat.name .. ": " .. stat.value) end
				if string.match(wndItemText, stat.name) == "MultiHit" then wnd:SetText(stat.name .. ": " .. stat.value) end
				if string.match(wndItemText, stat.name) == "Intensity" then wnd:SetText(stat.name .. ": " .. stat.value) end
				if string.match(wndItemText, stat.name) == "FocusReg" then wnd:SetText(stat.name .. ": " .. stat.value) end
				if string.match(wndItemText, stat.name) == "CritMit" then wnd:SetText(stat.name .. ": " .. stat.value) end
			end
		end
	end
end

function StatMeter:round(num, idp)
  local mult = 10^(idp or 0)
  return math.floor(num * mult + 0.5) / mult
end

function StatMeter:Pause()
	Apollo.StopTimer("UpdateTimer")
end

function StatMeter:Resume()
	Apollo.StartTimer("UpdateTimer")
end

-----------------------------------------------------------------------------------------------
-- StatMeter List Functions
-----------------------------------------------------------------------------------------------

function StatMeter:AddItem(i, statname, n) -- int index, str stat name, int stat value
	-- load the window item for the list item
	--Print(i..statname..n)
	local wnd = Apollo.LoadForm(self.xmlDoc, "StatItem", self.wndItemList, self)
	--Print("i = "..i)
	-- keep track of the StatItem created
	self.tItems[i] = wnd
	--Print(self.tItems[i]:GetName())
	--Print(wnd:GetName())
	--local children = wnd:GetChildren()
	--for key,child in ipairs(children) do
	--	Print(child)
	--end
	
	-- Setup the StatItem 
	--local wndItemText = wnd:FindChild("Text")
	--if wndItemText then -- make sure the text wnd exist
	--	wndItemText:SetText(statname .. ": " .. n) -- set the item wnd's text to statname
	--end
	if wnd then
		wnd:SetText(statname .. ": " .. n)
		wnd:SetFont("CRB_Interface12_BBO")
	end
	--print(wnd)
	wnd:SetData(i)
end

function StatMeter:PopulateStatList()
	local rSelected = "DPS"
	-- make sure the item list is empty to start with
	self:DestroyItemList()

	-- Fetch Player Current Stats
	local statset = self:GetAllStats()	
	
	--Import stats
	i = 0
	for key,item in pairs(statset) do
		if rSelected == "DPS" then
			i = i + 1
			--Print(item.name .. "  " .. item.value)
			--self:AddItem(i, item.name, item.value)
		end
	end
		
	-- now all the item are added, call ArrangeChildrenVert to list out the list items vertically
	self.wndItemList:ArrangeChildrenVert()
end

function StatMeter:PopulateSpecificStatList(roleid)
	-- make sure the item list is empty to start with
	self:DestroyItemList()
	
	--ToggleWnd
	local rollToggleWnd = self.wndMain:FindChild("RollToggle")
	
	-- Fetch Player Current Stats
	local statset = self:GetAllStats()	
	
	--Import stats
		--DPS = 0,
		--Heal = 1,
		--Tank = 2,
		--PVPHeal = 3,
		--PVPDPS = 4,
		--PVPTank = 5
	i = 0
	if statset then
		for key,stat in pairs(statset) do
			--DPS
			if roleid == 0 then
				i = i + 1
				rollToggleWnd:SetText("DPS")
				if stat.name == "AP" then self:AddItem(i, stat.name, stat.value) end
				if stat.name == "Critical" then self:AddItem(i, stat.name, stat.value) end
				if stat.name == "StrkThru" then self:AddItem(i, stat.name, stat.value) end
				if stat.name == "CritSev" then self:AddItem(i, stat.name, stat.value) end
				if stat.name == "MultiHit" then self:AddItem(i, stat.name, stat.value) end
			end
			--Healer
			if roleid == 1 then
				rollToggleWnd:SetText("Healer")
				i = i + 1
				if stat.name == "SP" then self:AddItem(i, stat.name, stat.value) end
				if stat.name == "HealOut" then self:AddItem(i, stat.name, stat.value) end	
				if stat.name == "Crit" then self:AddItem(i, stat.name, stat.value) end
				if stat.name == "Intensity" then self:AddItem(i, stat.name, stat.value) end
				if stat.name == "FocusReg" then self:AddItem(i, stat.name, stat.value) end
				if stat.name == "Armor" then self:AddItem(i, stat.name, stat.value) end
			end
			--Tank
			if roleid == 2 then
				rollToggleWnd:SetText("Tank")
				i = i + 1
				if stat.name == "SP" then self:AddItem(i, stat.name, stat.value) end
				if stat.name == "Deflect" then self:AddItem(i, stat.name, stat.value) end
				if stat.name == "DeflectC" then self:AddItem(i, stat.name, stat.value) end
				if stat.name == "CritMit" then self:AddItem(i, stat.name, stat.value) end
				if stat.name == "Armor" then self:AddItem(i, stat.name, stat.value) end
			end
			--PVPDps
			if roleid == 3 then 
				rollToggleWnd:SetText("PVP DPS")
				i = i + 1
				if stat.name == "AP" then self:AddItem(i, stat.name, stat.value) end
				if stat.name == "Critical" then self:AddItem(i, stat.name, stat.value) end
				if stat.name == "StrkThru" then self:AddItem(i, stat.name, stat.value) end
				if stat.name == "PVPOff" then self:AddItem(i,stat.name, stat.value) end
				if stat.name == "Armor" then self:AddItem(i, stat.name, stat.value) end
			end
			--PVP Healer
			if roleid == 4 then 
				rollToggleWnd:SetText("PVP Healer")
				i = i + 1	
				if stat.name == "SP" then self:AddItem(i, stat.name, stat.value) end
				if stat.name == "HealOut" then self:AddItem(i, stat.name, stat.value) end
				if stat.name == "Critical" then self:AddItem(i, stat.name, stat.value) end
				if stat.name == "Armor" then self:AddItem(i, stat.name, stat.value) end
				if stat.name == "PVPHeal" then self:AddItem(i,stat.name, stat.value) end
			end
			--PVP Tank
			if roleid == 5 then
				rollToggleWnd:SetText("PVP Tank")
				i = i + 1
				if stat.name == "SP" then self:AddItem(i, stat.name, stat.value) end
				if stat.name == "Deflect" then self:AddItem(i, stat.name, stat.value) end
				if stat.name == "DeflectC" then self:AddItem(i, stat.name, stat.value) end
				if stat.name == "Armor" then self:AddItem(i, stat.name, stat.value) end
				if stat.name == "PVPDef" then self:AddItem(i,stat.name, stat.value) end
			end
		end
	end
	-- now all the item are added, call ArrangeChildrenVert to list out the list items vertically
	self.wndItemList:ArrangeChildrenVert()
end

function StatMeter:DestroyItemList()
	-- destroy all the wnd inside the list
	--Print((self.tItems[3]:GetName()))
	if self.tItems ~= nil then
		for idx,wnd in pairs(self.tItems) do
			--Print(wnd:GetName())
			wnd:Destroy()
		end
	end
	--if self.tItems == nil then Print("table empty") end
	--if self.tItems ~= nil then Print("table contains data") end
	-- clear the list item array
	self.tItems = {}
end

function StatMeter:OnTest()
	Print("Test Time")
	if self.tItems == nil then Print("table empty") end
	if self.tItems ~= nil then Print("table contains data") end
end

-----------------------------------------------------------------------------------------------
-- StatMeterForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function StatMeter:OnOK()
	self.wndMain:Close() -- hide the window
end

-- when the Cancel button is clicked
function StatMeter:OnCancel()
	self.wndMain:Close() -- hide the window
end

function StatMeter:OnSave(eType)
	local tSave = {}
	if eType == GameLib.CodeEnumAddonSaveLevel.Character then	
		tSave = self.tConfig
	end
	return tSave
end

function StatMeter:OnRestore(type, savedData)
	if type == GameLib.CodeEnumAddonSaveLevel.Character then
		if savedData ~= nil and
		savedData.Version ~= nil and 
		savedData.Version == 1 then
			self.tConfig = savedData
		end
	end
end

function StatMeter:OnWindowMove( wndHandler, wndControl, nOldLeft, nOldTop, nOldRight, nOldBottom )
 	self.tConfig.Position = self:GetPosition(self.wndMain)
end

function StatMeter:GetPosition(form)
 	local result = nil
 	if form ~= nil then
  		local leftAnchor, topAnchor, rightAnchor, bottomAnchor = form:GetAnchorOffsets()
  		result =
  		{
   		Bottom = bottomAnchor,
   		Left = leftAnchor,
   		Right = rightAnchor,
   		Top = topAnchor,
  		}
 	end
 	return result
end


-----------------------------------------------------------------------------------------------
-- StatMeter Instance
-----------------------------------------------------------------------------------------------
local StatMeterInst = StatMeter:new()
StatMeterInst:Init()
